@@ -7,10 +7,11 @@
 //! In the future, this may also evolve into parsing the tar
 //! stream in Rust, not in C.
 
-use crate::cmdext::CommandRedirectionExt;
 use crate::Result;
 use anyhow::{anyhow, Context};
 use camino::{Utf8Component, Utf8Path, Utf8PathBuf};
+use cap_std_ext::cmdext::CapStdExtCommandExt;
+use cap_std_ext::rustix;
 use ostree::gio;
 use ostree::prelude::FileExt;
 use rustix::fd::FromFd;
@@ -126,7 +127,10 @@ fn normalize_validate_path(path: &Utf8Path) -> Result<NormalizedPathResult<'_>>
 /// Remember that we're parsing this while we're downloading it, and in order
 /// to verify integrity we rely on the total sha256 of the blob, so all content
 /// written before then must be considered untrusted.
-fn filter_tar(src: impl std::io::Read, dest: impl std::io::Write) -> Result<BTreeMap<String, u32>> {
+pub(crate) fn filter_tar(
+    src: impl std::io::Read,
+    dest: impl std::io::Write,
+) -> Result<BTreeMap<String, u32>> {
     let src = std::io::BufReader::new(src);
     let mut src = tar::Archive::new(src);
     let dest = BufWriter::new(dest);
@@ -152,7 +156,23 @@ fn filter_tar(src: impl std::io::Read, dest: impl std::io::Write) -> Result<BTre
         };
 
         let mut header = entry.header().clone();
-        dest.append_data(&mut header, normalized, entry)?;
+
+        // Need to use the entry.link_name() not the header.link_name()
+        // api as the header api does not handle long paths:
+        // https://github.com/alexcrichton/tar-rs/issues/192
+        match entry.header().entry_type() {
+            tar::EntryType::Link | tar::EntryType::Symlink => {
+                let target = entry.link_name()?.ok_or_else(|| anyhow!("Invalid link"))?;
+                let target = target
+                    .as_os_str()
+                    .to_str()
+                    .ok_or_else(|| anyhow!("Non-utf8 link"))?;
+                dest.append_link(&mut header, &normalized, target)?;
+            }
+            _ => {
+                dest.append_data(&mut header, normalized, entry)?;
+            }
+        }
     }
     dest.into_inner()?.flush()?;
     Ok(filtered)
